@@ -27,7 +27,7 @@
 // Copyright 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
 // Copyright 2022 Alexander Sulfrian <asulfrian@zedat.fu-berlin.de>
-// Copyright 2023, 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright 2024 Pratham Gandhi <ppg.1382@gmail.com>
 //
 //========================================================================
@@ -125,7 +125,7 @@ public:
     void setPartialName(const GooString &name);
     const GooString *getAlternateUiName() const;
     const GooString *getMappingName() const;
-    GooString *getFullyQualifiedName();
+    const GooString *getFullyQualifiedName();
 
     bool isModified() const;
 
@@ -142,8 +142,8 @@ public:
     static void decodeID(unsigned id, unsigned *pageNum, unsigned *fieldNum);
 
     void createWidgetAnnotation();
-    AnnotWidget *getWidgetAnnotation() const { return widget; }
-    void setWidgetAnnotation(AnnotWidget *_widget) { widget = _widget; }
+    std::shared_ptr<AnnotWidget> getWidgetAnnotation() const { return widget; }
+    void setWidgetAnnotation(std::shared_ptr<AnnotWidget> _widget) { widget = std::move(_widget); }
 
     virtual void updateWidgetAppearance() = 0;
 
@@ -152,7 +152,7 @@ public:
 protected:
     FormWidget(PDFDoc *docA, Object *aobj, unsigned num, Ref aref, FormField *fieldA);
 
-    AnnotWidget *widget;
+    std::shared_ptr<AnnotWidget> widget;
     FormField *field;
     FormFieldType type;
     Object obj;
@@ -195,7 +195,7 @@ public:
 
 protected:
     FormFieldButton *parent() const;
-    GooString *onStr;
+    std::unique_ptr<GooString> onStr;
 };
 
 //------------------------------------------------------------------------
@@ -210,9 +210,9 @@ public:
     const GooString *getContent() const;
 
     // expects a UTF16BE string
-    void setContent(const GooString *new_content);
+    void setContent(std::unique_ptr<GooString> new_content);
     // sets the text inside the field appearance stream
-    void setAppearanceContent(const GooString *new_content);
+    void setAppearanceContent(std::unique_ptr<GooString> new_content);
 
     void updateWidgetAppearance() override;
 
@@ -249,7 +249,7 @@ public:
     const GooString *getExportVal(int i) const;
     // select the i-th choice
     void select(int i);
-    void setAppearanceChoiceContent(const GooString *new_content);
+    void setAppearanceChoiceContent(std::unique_ptr<GooString> new_content);
     // toggle selection of the i-th choice
     void toggle(int i);
 
@@ -258,7 +258,7 @@ public:
 
     // except a UTF16BE string
     // only work for editable combo box, set the user-entered text as the current choice
-    void setEditChoice(const GooString *new_content);
+    void setEditChoice(std::unique_ptr<GooString> new_content);
 
     const GooString *getEditChoice() const;
 
@@ -375,7 +375,7 @@ public:
     void setPartialName(const GooString &name);
     const GooString *getAlternateUiName() const { return alternateUiName.get(); }
     const GooString *getMappingName() const { return mappingName.get(); }
-    GooString *getFullyQualifiedName();
+    const GooString *getFullyQualifiedName() const;
 
     FormWidget *findWidgetByRef(Ref aref);
     int getNumWidgets() const { return terminal ? widgets.size() : 0; }
@@ -418,7 +418,7 @@ protected:
     std::unique_ptr<GooString> partialName; // T field
     std::unique_ptr<GooString> alternateUiName; // TU field
     std::unique_ptr<GooString> mappingName; // TM field
-    GooString *fullyQualifiedName;
+    mutable std::unique_ptr<GooString> fullyQualifiedName;
 
     // Variable Text
     std::unique_ptr<GooString> defaultAppearance;
@@ -490,8 +490,8 @@ public:
 
     const GooString *getContent() const { return content.get(); }
     const GooString *getAppearanceContent() const { return internalContent ? internalContent.get() : content.get(); }
-    void setContentCopy(const GooString *new_content);
-    void setAppearanceContentCopy(const GooString *new_content);
+    void setContent(std::unique_ptr<GooString> new_content);
+    void setAppearanceContent(std::unique_ptr<GooString> new_content);
     ~FormFieldText() override;
 
     bool isMultiline() const { return multiline; }
@@ -549,7 +549,7 @@ public:
     const GooString *getSelectedChoice() const;
     const GooString *getAppearanceSelectedChoice() const { return appearanceSelectedChoice ? appearanceSelectedChoice.get() : getSelectedChoice(); }
 
-    void setAppearanceChoiceContentCopy(const GooString *new_content);
+    void setAppearanceChoiceContent(std::unique_ptr<GooString> new_content);
 
     // select the i-th choice
     void select(int i);
@@ -561,7 +561,7 @@ public:
     void deselectAll();
 
     // only work for editable combo box, set the user-entered text as the current choice
-    void setEditChoice(const GooString *new_content);
+    void setEditChoice(std::unique_ptr<GooString> new_content);
 
     const GooString *getEditChoice() const;
 
